/*
  controllers.js
  Angular module that provides the main and only controller for the wesco airbox app
*/


var airboxControllers = angular.module('airboxControllers', ['airboxDSS','dSS']);

airboxControllers.controller('AirboxConfCtrl',
			     ['$q','$scope','$modal','$timeout','airboxDSS','dss','gettextCatalog','$confirm',
			      function($q,$scope,$modal,$timeout,airboxDSS,dss,gettextCatalog,$confirm) {


				  $scope.loadAll = function() {
				      var promises = [];
				      promises.push($scope.refreshAirboxes());
				      return $q.all(promises);
				  };



				  // Ajax request states to show/hide elements depending on if content is loaded
				  $scope.ajax = {
				      'BusyFlag': false,
				  };


				  // Initialize variables database
				  $scope.pages = {};

				  airboxDSS.getAirboxes().then(function(airboxes){
				      for (var i=0; i< airboxes.length;i++){
					  var description=airboxes[i].Description;
					  var dsid=airboxes[i].dSID;
					  var key="k_"+airboxes[i].dSID;
					  //$scope.pages.push({"name":dsid,"dsid":dsid,"description":description,"variables":Airbox_GetVars()});
					  $scope.pages[key]={"name":dsid,"dsid":dsid,"description":description,"variables":Airbox_GetVars(),"all_selected":false};


				      };
				      dss.Property.getString("/system/language/locale").then(function(locale) {
					  gettextCatalog.setCurrentLanguage(locale.substr(0,2));
				      });

				  }).then(function(){
				      $scope.varindex = {};
				      for (var ip in $scope.pages) { // variable cathegories
				      	  for (var iv in $scope.pages[ip].variables) { // variables in each cathegory
				      	      var v = $scope.pages[ip].variables[iv];
				      	      // if($scope.varindex[v.name]) {
				      	      //     console.log("WARNING: Multiple variables with name "+v.name+" found in definition file.");
				      	      // }
				      	      //$scope.varindex[v.name] = v; // store references to access by name
				      	      v.selected=false;
				      	      if(typeof v.uservalue==='undefined') {
				      		  v.uservalue = v.default;
				      	      }
				      	  };
				      };
				  });


				  $scope.selectAll=function(dsid){
				      // Select all configuration registers for a given airbox

				      var key="k_"+dsid;
				      var all_selected=$scope.pages[key].all_selected;
				      $scope.pages[key].all_selected=!all_selected;
				      for (var iv in  $scope.pages[key].variables) { // variables in each cathegory
					  var v = $scope.pages[key].variables[iv];
					  v.selected=!all_selected;
				      }
				  };


				  $scope.readSelectedRegisters=function(dsid){
				      // Read selected registers on the corresponding aribox

				      var title;
				      var text;

				      failed=function()
				      {title=gettextCatalog.getString("Error")
				       text=gettextCatalog.getString("The reading operation could not be completed! Please verify the connections and retry later.");
				       $confirm({text:text, title: title, ok: 'OK',cancel:'Cancel'});
				      }

				      airboxDSS.getBusyFlag().then(function(Busy){
					  if (Busy){
					      title=gettextCatalog.getString("Channel is busy")
					      text=gettextCatalog.getString("Reading or writing operation in progress, please retry later.")
					      $confirm({text:text, title: title, ok: 'OK',cancel:'Cancel'})
						  .then(function() {
						      return;
						  });
					      return;
					  }

					  var key="k_"+dsid;
					  var regs=[]; // choose the selected (checked) registers
					  for (var iv in $scope.pages[key].variables) {
					      var v = $scope.pages[key].variables[iv];
					      if (v.selected){
						  regs.push(v.reg); // this registers contains the checked variables' registers
					      }
					  }


					  if (regs.length > 0){

					      var estimated_time=5*regs.length;
					      var eventID=43; // arbitrary ID for the event subscription
					      var eventTimeout=8*estimated_time*1000;//ms

					      title=gettextCatalog.getString("Do you want to continue?");
					      text=gettextCatalog.getString("This operation might take a while (approx: {{seconds}} s). Please do not refresh or close this page.)",{seconds: estimated_time });
					      $confirm({text:text, title: title, ok: 'OK',cancel:'Cancel'})
						  .then(function() {
						      $scope.ajax.ButtonsLocked=true;


						      //$scope.ajax.airbox_read_regs = true;
						      //airboxDSS.readRegs(dsid,regs); // Read the registers

						      return  airboxDSS.readRegs(dsid,regs).then(function(result){
							  return dss.Event.unsubscribe('wesco_RegistersOpCompleted',eventID).then(function(result){
							      return dss.Event.subscribe('wesco_RegistersOpCompleted',eventID).then(function(result){
								  return dss.Event.get(eventID,eventTimeout).then(function(result){
								      //     //[{"events":[{"name":"wesco_RegistersOpCompleted",
								      //"properties":{"Dict":"{\"0\":{\"dSID\":\"302ed89f43f00d0000099618\",
								      //\"Path\":\"/scripts/wesco/Airboxes/Devices/302ed89f43f0000000000d000009961800\",
								      //\"Ntries\":3,\"completed\":[200,101,139]}}","Regs":"[12,13,15]"},
								      //"source":{"set":".zone(0).group(0)","groupID":0,"zoneID":0,
								      //"isApartment":true,"isGroup":false,"isDevice":false}}]}]

								      $scope.ajax.ButtonsLocked = false;

								      if(result.events[0]!==undefined){
									  if (result.events[0].properties!==undefined){
							  		      var Dict=JSON.parse(result.events[0].properties.Dict);
							  		      var regs=JSON.parse(result.events[0].properties.Regs);

							  		      if (Dict[0]){
							  			  var values=Dict[0].completed;
							  		      }
							  		      if(values !==undefined && regs!==undefined){
							  			  if(values.length==regs.length && values.length>0){
							  			      // update variables with returned values
							  			      for (var i=0;i< regs.length;i++){
							  				  var var_index="REG_"+regs[i];
											  if($scope.pages[key].variables[var_index].unit=='%'){
							  				      $scope.pages[key].variables[var_index].uservalue=values[i]-100;
											  }else if(typeof $scope.pages[key].variables[var_index].values!=='undefined')
											  { // Selectable values require strings
											      $scope.pages[key].variables[var_index].uservalue=String(values[i]);
											  }else{
											      $scope.pages[key].variables[var_index].uservalue=values[i]-100;
											  }
										      }


										      title=gettextCatalog.getString("Success")
										      text=gettextCatalog.getString("{{N}} registers successfully read from airbox {{dsid}}!",{N:values.length-1,dsid:dsid});
										      $confirm({text:text, title: title, ok: 'OK',cancel:'Cancel'});
							  			  }
							  		      }else{
										  failed();
									      }
									  }else{
									      failed();
									  }
								      }else{
									  failed();
							  	      }
								      // $scope.ajax.airbox_read_regs = false;
								  });
							      }).catch(function(e){
								  console.log(e);
								  failed();
							      });
							  }).catch(function(e){
							      console.log(e);
							      failed();
							  });
						      });
						  });
					  } // end if regs.length
				      }).catch(function(e){
					  console.log(e);
					  failed();
				      });
				  };





				  $scope.writeSelectedRegisters=function(dsid,toAll){
				      // write selected registers on the corresponding aribox or to all airboxes
				      // The Boolean toAll indicates whether the values are to be written to all airboxes (true) or just to the one indicated by dsid.

				      var title;
				      var text;

				      failed=function(){
					  title=gettextCatalog.getString("Error")
					  text=gettextCatalog.getString("The writing operation could not be completed!");
					  $confirm({text:text, title: title, ok: 'OK',cancel:'Cancel'});
					  $scope.ajax.ButtonsLocked = false;

				      }

				      airboxDSS.getBusyFlag().then(function(Busy){
					  if (Busy){
					      title=gettextCatalog.getString("Channel is busy")
					      text=gettextCatalog.getString("Reading or writing operation in progress, please retry later.")
					      $confirm({text:text, title: title, ok: 'OK',cancel:'Cancel'})
						  .then(function() {
						      return;
						  });
					      return;
					  }


					  var key="k_"+dsid;
					  var regs=[]; // choose the selected (checked) registers
					  for (var iv in $scope.pages[key].variables) {
					      var v = $scope.pages[key].variables[iv];
					      if (v.selected){
						  if(v.unit=='%'){
						      regs.push([v.reg,parseInt(v.uservalue)+100]); // this registers contains the checked variables' registers
						  }else{
						      regs.push([v.reg,v.uservalue]); // this registers contains the checked variables' registers
						  }
					      }
					  }
					  var eventID=47; // arbitrary ID for event registration
					  var eventTimeout=300000;//ms

					  if (regs.length > 0){

					      title=gettextCatalog.getString("Do you want to continue?");
					      text=gettextCatalog.getString("This operation might take several minutes");
					      $confirm({text:text, title: title, ok: 'OK',cancel:'Cancel'})
						  .then(function(){

						      $scope.ajax.ButtonsLocked = true;

						      regs.push([3,4]); // save parameters to flash

						      $scope.ajax.airbox_read_regs = true;
						      //airboxDSS.readRegs(dsid,regs); // Read the registers

						      return  airboxDSS.writeRegs(dsid,regs).then(function(result){
							  return dss.Event.unsubscribe('wesco_RegistersOpCompleted',eventID).then(function(result){
							      return dss.Event.subscribe('wesco_RegistersOpCompleted',eventID).then(function(result){
								  return dss.Event.get(eventID,eventTimeout).then(function(result){

								      $scope.ajax.ButtonsLocked = false;

								      if(result.events[0]!==undefined){
									  if (result.events[0].properties!==undefined){
							  		      var Dict=JSON.parse(result.events[0].properties.Dict);
							  		      var regs=JSON.parse(result.events[0].properties.Regs);
									      //console.log(regs);

									      title=gettextCatalog.getString("Success")
									      text=gettextCatalog.getString("{{N}} registers successfully transmitted!",{N:regs.length-1});
									      $confirm({text:text, title: title, ok: 'OK',cancel:'Cancel'});

									  }else{
									      failed();
									  }
								      }else{
									  failed();
								      }
								      // $scope.ajax.airbox_read_regs = false;
								  });

							      }).catch(function(e){
								  console.log(e);
								  failed();
							      });
							  }).catch(function(e){
							      console.log(e);
							      failed();
							  });
						      });
						  });
					  }
				      }).catch(function(e){
					  console.log(e);
					  failed();
				      });


				  };// END of $scope.writeSelectedRegisters=function(dsid,toAll){


				  $scope.importVarsFromFile = function(dsid){

				      var selectModal = $modal.open({
					  animation: true,
					  templateUrl: 'importSelectModal.html',
					  controller: 'importSelectModalCtrl',
					  size: '',
				      });

				      selectModal.result.then(function (selection) {
	    				  try {
	    				      $scope.importLog = importVars(selection.data,dsid);
					      var resultModal = $modal.open({
						  animation: true,
						  templateUrl: 'importResultModal.html',
						  controller: 'importResultsModalCtrl',
						  size: '',
						  resolve: {
			      			      'importLog': function() { return $scope.importLog; },
						  }
					      });
					  }
					  catch(err) {
					      alert(err);
					  }
				      });
				  }; // END of $scope.importVarsFromFile = function(dsid){

				  $scope.exportVarsToFile = function(dsid) {
				      try {
					  var contents = "";
					  var key="k_"+dsid;
					  contents+='#Airbox\t'+dsid+'\n#Reg\tValue\n';
					  for (var iv in $scope.pages[key].variables) {
					      var v = $scope.pages[key].variables[iv];
					      var reg= v.reg;
					      var value=v.uservalue;
					      contents+=reg+'\t'+value+'\n';
					  }

					  var blob = new Blob([contents], {type: "text/plain;charset=utf-8"});
					  saveAs(blob, "Wesco_Airbox_"+dsid+".txt");
				      }
				      catch(err) {
					  alert(err);
				      }
				  }; // END of  $scope.exportVarsToFile = function() {



				  function importVars (variables,dsid,setdefaults) {
				      var importLog = {
    					  'changed':[],
    					  'unchanged':[],
    					  'undefined':[],
    					  'invalid':[],
    				      };


				      var key="k_"+dsid;
				      for (var iv in variables) {
					  var reg_index="REG_"+variables[iv].reg; // retrieve variable register number
					  var v = $scope.pages[key].variables[reg_index];

					  if(typeof v === 'undefined') {
					      importLog['undefined'].push(variables[iv].reg);
					  }
					  else {
					      var newvalue;
					      if(typeof v.values=='undefined') { // if it is a value with no options
						  // regular continous value
						  switch(v.type.toLowerCase()) {
						  case "uint8":
						      newvalue = parseInt(variables[iv].value);
						      break;

						  default: // for the time being all variables are uint8
						      newvalue = parseInt(variables[iv].value);
						      break;
						  }
					      }
					      else {
						  // values stored with string index
						  newvalue = variables[iv].value;
					      }
					      if(newvalue>v.max||newvalue<v.min) {
						  // verify range
						  importLog['invalid'].push(variables[iv].reg);
					      }
					      else {
						  if(newvalue != v.uservalue) {
						      importLog['changed'].push(variables[iv].reg);
						      v.uservalue = newvalue;
						  }
						  else {
						      importLog['unchanged'].push(variables[iv].reg);
						  }
						  // Overwrite defaults
						  if(setdefaults) {
						      v.default = newvalue;
						  }
					      }
					  }
				      }
				      return importLog;
				  };// END of function importVars


			      }]); // end controller AirboxConf





// main application controller
airboxControllers.controller('OverviewCtrl',
			     ['$q','$scope','$timeout','airboxDSS','dss','gettextCatalog',
			      function($q,$scope,$timeout,airboxDSS,dss,gettextCatalog) {

				  $scope.removeVentilationScheduleRow = function (idx) {
				      $scope.ventilation_schedule.rules.splice(idx, 1);
				  };

				  $scope.addVentilationScheduleRow = function () {
				      var rule={};
				      rule['day']=$scope.ventilation_schedule.day.uservalue;
				      rule['hour']=$scope.ventilation_schedule.hour.uservalue;
				      rule['Presence']=$scope.ventilation_schedule.level.uservalue_presence;
				      rule['Absence']=$scope.ventilation_schedule.level.uservalue_absence;
				      $scope.ventilation_schedule.rules.push(rule);
				  };

				  $scope.saveVentilationSchedule = function () {
				      var schedule={};
				      for (var r in $scope.ventilation_schedule.rules){
					  var rule=$scope.ventilation_schedule.rules[r];

					  var day=rule.day;
					  var hour=rule.hour;
					  var values=[rule.Presence,rule.Absence];
					  if (!schedule[day]){
					      schedule[day]={};
					  }
					  schedule[day][hour]=values;
				      }
				      return airboxDSS.configVentilationSchedule(schedule);

				  };




				  $scope.initializeVentilationSchedule = function(){
				      // Airbox schedule configuration
				      var ventilation_schedule={
					  'rules':[],
					  'day':{'uservalue':'7','values':{
					      '0':"Sunday",
					      '1':"Monday",
					      '2':"Tuesday",
					      '3':"Wednesday",
					      '4':"Thursday",
					      '5':"Friday",
					      '6':"Saturday",
					      '7':"Weekday",
					      '8':"Weekend",
					  }},
					  'hour':{'uservalue':'0','values':['0','1','2','3','4','5','6','7','8','9','10','11','12','13','14','15','16','17','18','19','20','21','22','23']
						 },
					  'level':{'uservalue_presence':'2','uservalue_absence':'2','values':{
					      '2':"Off",
					      '3':"Low",
					      '4':"Medium",
					      '5':"Nominal",
					      '6':"Full",
					  }}
				      };
				      //

				      $scope.ventilation_schedule=ventilation_schedule;
				      airboxDSS.getVentilationSchedule().then(function(rules){
				      	  $scope.ventilation_schedule.rules=rules;
				      });

				  };


				  // Global ajax error message to display
				  $scope.ajaxError = false;


				  // Ajax request states to show/hide elements depending on if content is loaded
				  $scope.ajax = {
				      'levelReady': false,
				      'settingLevel': false,
				      'settingConfig': false,
				      'exhaustFlapCandidatesReady': false,
				      'pushTargetOptionsReady': false,
				      'configReady': false,
				      'errorlogReady': false,
				      'deviceListReady': false,
				  };

				  // Variables for pagination of logfiles
				  $scope.logTable = {
				      'maxSize' : 10,
				      'currentPage' : 1,
				  };

				  // Translations of error texts
				  $scope.airbox_error_texts = {
				      'NOERROR':'OK',
				      'POWERERR':'Spannungsstörung',
				      'SENSORERR':'Sensorstörung',
				      'FANERR':'Ventilatorstörung',
				      'FLAPERR':'Klappenstörung',
				      'FROSTERR':'Froststörung',
				      'FROSTWAR':'Frostwarnung',
				      'NEWAIRBOX':'Neue Airbox erkannt',
				      'AIRBOXLOST':'Airbox nicht mehr vorhanden',
				      'TMOUTERR':'Verbindungsfehler',
				  };

				  $scope.refreshCurrentLevel = function() {
				      $scope.ajax.levelReady = false;
				      return airboxDSS.getCurrentLevel().then(function(currentLevel) {
					  $scope.currentLevel = currentLevel;
					  $scope.ajax.levelReady = true;
				      });
				  };

				  $scope.refreshConfig = function() {
				      $scope.ajax.configReady = false;
				      return airboxDSS.getConfig().then(function(config) {
					  $scope.config = config;
					  $scope.selectedPushTargets = {};
					  for (var i = 0; i < $scope.config.PushTargets.length; i++) {
					      $scope.selectedPushTargets[$scope.config.PushTargets[i]] = true;
					  }
					  $scope.ajax.configReady = true;
				      });
				  };

				  $scope.refreshErrorLog = function() {
				      $scope.ajax.errorlogReady = false;
				      return airboxDSS.getErrorLog().then(function(errorlog) {
					  $scope.errorLog = [];
					  for (var i = 0; i < errorlog.length; i++) {
					      $scope.errorLog.push({
						  'datetime':errorlog[i][0],
						  'type':errorlog[i][1],
						  'dSID':errorlog[i][2],
						  'name':$scope.Airboxes.nameById[errorlog[i][2]]?$scope.Airboxes.nameById[errorlog[i][2]]:errorlog[i][2],
						  'errorcode':errorlog[i][3],
					      });
					  }

					  $scope.ajax.errorlogReady = true;
				      });
				  };

				  $scope.refreshExhaustFlapCandidates = function() {
				      $scope.ajax.exhaustFlapCandidatesReady = false;
				      return airboxDSS.getExhaustFlapCandidates().then(function(devlist) {
					  $scope.exhaustFlapCandidates = devlist;
					  $scope.ajax.exhaustFlapCandidatesReady = true;
				      })
				  };

				  $scope.refreshPushTargetOptions = function() {
				      $scope.ajax.pushTargetOptionsReady = false;
				      return airboxDSS.refreshPushTargetOptions().then(function() {
					  return $timeout(function() {
					      return airboxDSS.getPushTargetOptions().then(function(targets) {
						  $scope.pushTargetOptions = targets;
						  $scope.ajax.pushTargetOptionsReady = true;
					      })
					  },3000);
				      });
				  };

				  $scope.clearAllAirboxErrors = function() {
				      var answer = confirm("Achtung: Nach der Fehlerbehebung müssen die betroffenen Airboxen neugestartet werden. Wenn die Störung weiterhin besteht ist dies nach dem Entfernen der Störungsmeldungen nicht sofort ersichtlich. Bitte überprüfen Sie in 30 Minuten ob der Fehler nun behoben ist.")
				      if (!answer) {
					  $scope.refreshConfig();
					  return;
				      }

				      $scope.ajax.deviceListReady = false;
				      return airboxDSS.clearAllAirboxErrors().then(function() {
					  return $timeout(function() {
					      return $scope.refreshAirboxes();
					  },3000);
				      });
				  };

				  $scope.clearAirboxError = function(dSUID) {
				      var answer = confirm("Introduce message here!! The timeout and reload can be enough as feedback mechanism. Evaluate weather an event-based feedback mechanism is required");
				      if (!answer) {
					  $scope.refreshConfig();
					  return;
				      }

				      return airboxDSS.clearAirboxError(dSUID).then(function(){
					  return $timeout(function() {
					      return $scope.refreshAirboxes();
					  },3000);
				      });
				  };


				  $scope.updateConfig = function() {
				      // var answer = confirm("Drücken Sie OK um die Konfiguration zu speichern.")
				      // if (!answer) {
				      // 	  $scope.refreshConfig();
				      // 	  return;
				      // }

				      var promises = [];
				      $scope.ajax.settingConfig = false;

				      // Create list of selected UUIDs
				      $scope.config.PushTargets = [];
				      for (var UUID in $scope.selectedPushTargets) {
					  if ($scope.selectedPushTargets.hasOwnProperty(UUID)) {
					      if($scope.selectedPushTargets[UUID]) {
						  $scope.config.PushTargets.push(UUID);
					      }
					  }
				      }

				      promises.push(airboxDSS.setPushTargets($scope.config.PushTargets));
				      //	promises.push(airboxDSS.setExhaustFlap($scope.config.ExhaustFlap));
				      promises.push(airboxDSS.setEmailAddress($scope.config.EmailAddress));

				      //@Diego
				      // promises.push(dss.Scene.disableScene($scope.config.ExhaustFlap,72)); // Scene 72 -> leave home
				      // promises.push(dss.Scene.disableScene($scope.config.ExhaustFlap,69)); // Scene 69 -> sleeping
				      promises.push(airboxDSS.storeConfigurableParameters());


				      return $q.all(promises).then(function() {
					  $scope.ajax.settingConfig = true;
				      });
				  };

				  //@Diego
				  $scope.updateExhaustFlap = function() {
				      var answer = confirm("Sind Sie sicher, dass sie die Abluftklappe-Konfiguration ändern möchten?")
				      if (!answer) {
					  $scope.refreshConfig();
					  return;
				      }

				      var promises = [];
				      $scope.ajax.settingConfig = false;

				      promises.push(airboxDSS.setExhaustFlap($scope.config.ExhaustFlap));

				      //@Diego
				      promises.push(dss.Scene.setJokerGroup($scope.config.ExhaustFlap,8)); // joker color 8 is black

				      promises.push(dss.Scene.disableScene($scope.config.ExhaustFlap,72)); // Scene 72 -> leave home
				      promises.push(dss.Scene.disableScene($scope.config.ExhaustFlap,69)); // Scene 69 -> sleeping
				      promises.push(dss.Scene.disableScene($scope.config.ExhaustFlap,68)); // Scene 68 -> deep off
				      promises.push(dss.Scene.disableScene($scope.config.ExhaustFlap,67)); // Scene 67 -> standby
				      promises.push(airboxDSS.storeExhaustFlap());


				      return $q.all(promises).then(function() {
					  $scope.ajax.settingConfig = true;
				      });
				  };


				  //@Diego
				  $scope.restartAirboxes = function() {
				      var promises = [];
				      promises.push(airboxDSS.restartAirboxes());
				  };


				  $scope.refreshAirboxes = function() {
				      var promises = [];
				      if(typeof $scope['Airboxes'] == 'undefined') {
					  $scope['Airboxes'] = {
					      'nameById': {},
					  };
				      }

				      $scope.ajax.deviceListReady = false;


				      airboxDSS.rediscoverAirboxes();
				      // Wait 5 seconds for background script to update property tree
				      return $timeout(function() {
					  promises.push(airboxDSS.getAirboxConfig().then(function(config) {
					      for (var attrname in config) {
						  $scope['Airboxes'][attrname] = config[attrname];
					      }
					  }));
					  promises.push(airboxDSS.getAirboxes().then(function(airboxes) {
					      $scope.Airboxes.Devices = airboxes;
					      $scope.Airboxes.Error = false;
					      // Create index to resolve name by dSUID
					      for (var i = 0; i < $scope.Airboxes.Devices.length; i++) {
						  if($scope.Airboxes.Devices[i].errorcode!='NOERROR') {
						      $scope.Airboxes.Error = true;
						  }
						  $scope.Airboxes.nameById[$scope.Airboxes.Devices[i].dSID] = $scope.Airboxes.Devices[i].Description;
					      };
					  }));


					  return $q.all(promises).then(function() {
					      $scope.ajax.deviceListReady = true;
					  });
				      },3000);
				  };

				  $scope.setLevel = function(level) {
				      $scope.ajax.settingLevel = true;
				      airboxDSS.setLevel(level).then(function() {
					  $scope.ajax.settingLevel = false;
				      });
				  };

				  $scope.initDSS = function() {
				      // get locale, if german set ui to german otherwise use english
				      return dss.Property.getString("/system/language/locale").then(function(locale) {
					  gettextCatalog.setCurrentLanguage(locale.substr(0,2));
					  //if(locale.substr(0,2)=="de") {
					  //  }
				      });
				  }

				  $scope.loadAll = function() {
				      var promises = [];
				      promises.push($scope.initDSS());
				      promises.push($scope.refreshConfig());
				      promises.push($scope.refreshAirboxes());
				      promises.push($scope.refreshExhaustFlapCandidates());
				      promises.push($scope.refreshPushTargetOptions());
				      promises.push($scope.initializeVentilationSchedule());
				      return $q.all(promises);
				  };
 				  (function pollLevel() {
 				      $scope.refreshCurrentLevel().then(function() {
					  $timeout(pollLevel, 3000);
				      });
				  })();
				  $scope.loadAll().then(function() {
				      return $scope.refreshErrorLog();
				  },
							function(err) {
							    $scope.ajaxError = err;
							    console.log(err);
							});

			      }]);


airboxControllers.controller('importSelectModalCtrl'
			     ,['$scope','$modalInstance','airboxDSS',
			       function ($scope, $modalInstance,airboxDSS) {
				   $scope.ok = function () {
				       var file = document.getElementById('selectedFile').files[0];
				       if (file.type.match(/text.*/)) {
					   var reader = new FileReader();
					   reader.onload = function(e) {
					       $modalInstance.close({
						   'source':'file',
						   'data': airboxDSS.parseVarFile(reader.result),
					       });
					   }
					   reader.readAsText(file);
				       }
				       else {
					   throw "ERROR: Could not read file. Only plain text files with tab-separated values can be imported.";
				       }

				   };
				   $scope.cancel = function () {
				       $modalInstance.dismiss('cancel');
				   };
			       }]);


airboxControllers.controller('importResultsModalCtrl',
			     function ($scope, $modalInstance, importLog) {
				 $scope.importLog = importLog;
				 $scope.ok = function() {
				     $modalInstance.close('ok');
				 }
			     });
